### Benchmark on complex puzzles (remove `2>/dev/null` to see the picture)

```
# use redirection tricks to swap outputs to grep only logs, not the picture
# https://stackoverflow.com/a/2381643/1177288

# http://webpbn.com/pbnsolve.html
for i in 1611 1694 6739 4645 2040 2712 6574 8098 2556; do
    echo "Solving PBN's puzzle #$i (http://webpbn.com/$i) ..."
    time python -m pynogram --pbn $i --timeout=3600 --max-solutions=2 --draw-final -v 3>&- 2>/dev/null 3>&1 1>&2 2>&3 |
        grep -i contradict
done

for i in football einstein MLP; do
    echo "Solving local puzzle $i ..."
    time python -m pynogram --board $i --draw-final -v 3>&- 2>/dev/null 3>&1 1>&2 2>&3 |
        grep -i contradict
done
```

Currently it gives these numbers on the fast (_Intel(R) Xeon(R) CPU E3-1275 v5 @ 3.60GHz_)
and slower (_Intel(R) Core(TM) i5 CPU  M 560  @ 2.67GHz_) CPUs:

| Name      | Fast CPU, sec | Slow CPU, sec | Contradictions found | Solution rate, % | Solutions found | Search depth |
|-----------|--------------:|--------------:|:--------------------:|-----------------:|----------------:|-------------:|
|-- webpbn.com --
| 1611      | 1.74          | 2.34          | 18                   | 100
| 1694      | 2.14          | 3.93          | 99                   | 100
| **6739**  | 2.90          | 5.05          | 95                   | 98.5625          | 2 | 1
| 4645      | 5.16          | 9.14          | 33                   | 100
| 2040      | 6.20          | 11.49         | 204                  | 100
| **2712**  | 716           | ~1430         | 149, 8               | 54.78 -> 55.05   | 2 | 11
| **6574**  | 866           | ~1730         | 44, 2                | 29.6 -> 100      | 1 | 7
| **8098**  | >1800         | >1800         | 0                    | 0                | 1 | 8
| **2556**  | >1800         | >1800         | 8                    | 92.72 -> 93.03   | 0 | 11
|-- Local --
| [football](../pynogram/examples/football.txt) | 0.41  | 0.52  | 1 (6)    | 100
| [einstein](../pynogram/examples/einstein.txt) | 1.42  | 1.96  | 0        | 100
| [MLP](../pynogram/examples/MLP.txt)           | 11.13 | 18.27 | 1 (429)  | 100



### Colored puzzles

```
for i in 11360 12924 13951 15711 2014 2021 2518 3408 5353 6098 7598 7959 8364 10266 10295 11059 11443 11924 12987 13932 13983 14040 14442 14691 14913 14918 15082 15233 15370; do
    echo "Solving PBN's puzzle #$i (http://webpbn.com/$i) ..."
    /usr/bin/time -f 'Total: %U' python -m pynogram --pbn ${i} --draw-final --max-solutions=2 -v 3>&- 2>/dev/null 3>&1 1>&2 2>&3 |
    grep -iP 'contradiction|Total'
    echo
done
```

| #         | Contradictions found | Solution rate, % | Time, sec | Solutions found | Search depth |
|-----------|:--------------------:|-----------------:|----------:|----------------:|-------------:|
| 11360     | 6                    | 100              |   0.66
| 12924     | 1                    | 100              |   0.11
| 13951     | 1                    | 100              |   0.10
| **15711** | 0                    | 84               |  14.26    | 2               | 27
| **2014**  | 0                    | 99.5             |   0.28    | 2               | 1
| 2021      | 1                    | 100              |   0.11
| 2518      | 12                   | 100              |   0.92
| 3408      | 2                    | 100              |   0.20
| **5353**  | 0                    | 96.6667          |   1.82    | 2               | 0
| 6098      | 1                    | 100              |   0.73
| **7598**  | 0                    | 83.6667          |   8.78    | 2               | 23
| 7959      | 1                    | 100              |   0.12
| 8364      | 1                    | 100              |   0.12
| **10266** | 0                    | 84               |   1.35    | 2               | 23
| **10295** | 4, 0                 | 98.4167          |   5.23    | 2               | 6
| **11059** | 0                    | 96               |   0.16    | 2               | 1
| **11443** | 0                    | 68               |   0.14    | 2               | 0
| 11924     | 1                    | 100              |   0.15
| 12987     | 1                    | 100              |   0.16
| 13932     | 1                    | 100              |   0.15
| 13983     | 1                    | 100              |   0.12
| 14040     | 1                    | 100              |   0.12
| 14442     | 1                    | 100              |   0.12
| **14691** | 0                    | 98.6667          |   0.54    | 2               | 0
| 14913     | 2                    | 100              |   0.30
| 14918     | 1                    | 100              |   0.17
| 15082     | 1                    | 100              |   0.13
| **15233** | 0                    | 98.8571          |   0.31    | 2               | 1
| 15370     | 1                    | 100              |   0.16



## Long-solved board (more than 5 minutes, the first 35000 of webpbn's puzzles):

```
$ bash tests/bench.sh {1..35000} 2>&1 >> bench.log &
# and wait for several hours...

$ while read t
  do
    id=$(grep -P ${t} ${log_file} -A2 | grep -oP '#\K(\d+)' | awk '{print $1-1}')
    echo "$id: $t"
  done < <(grep -oP 'Total: \K(.+)' bench.log | sort -gr | awk '$1 > 300')
```

| #         |  Solution rate, % | Time, sec | Solutions found | Search depth |
|-----------|:-----------------:|----------:|----------------:|-------------:|
| -- black and white:
| 607       | 58.56             | 9         | 4               | 2
| **2556**  | 92.72 -> 94.50    | 1895/43   | 3/2             | 10/50
| 2647      | 70.33             | 22        | 3               | 34
| **2712**  | 54.78 -> 55.05    | 294       | 2               | 7
| 2903      | 62.61 -> 100      | 4         | 1               | 1
| 3162      | 83.14 -> 100      | 7         | 1               | 2
| 3163      | 23.68 -> 100      | 29        | 1               | 2
| 3335      | 0 -> 100          | 2         | 1               | 2
| 3379      | 55.88 -> 100      | 10        | 1               | 2
| 3469      | 0 -> 100          | 2         | 1               | 2
| _3867_    | 87.10             | 11        | 2               | 14
| 5005      | 46.50 -> 100      | 1         | 1               | 1
| **6574**  | 29.6 -> 100       | 522       | 1               | 7
| 7290      | 56.6 -> 57.16     | 116       | 2               | 8
| 7382      | 62.84 -> 64.64    | 52        | 2               | 9
| **8098**  | 0 -> 38.50        | 1282      | 1               | 8
| **9892**  | 33.30 -> 33.35    | >3600     | 0               | 14-15
| **10088** | 23.60 -> 23.66    | 50        | 2               | 14
| 12534     | 35.87 -> 36.74    | 40        | 2               | 7
| **12548** | 18.56             | >3600     | 0               | 13
| 13480     | 12.68             | >3600     | 0               | 12
| 16141     | 88.91 -> 89.09    | 13        | 2               | 4
| 16435     | 24.67 -> 100      | 6         | 1               | 1
| 16900     | 48.06 -> 48.86    | >3600     | 0               | 10
| **18297** | 13.49 -> 14.15    | 2246      | 2               | 8
| 18980     | 76 -> 100         | 1         | 1               | 2
| 19080     | 00.12             | 128       | 4               | 103
| 20058     | 49.40             | 28        | 3               | 31
| 20761     | 25.11 -> 100      | 6         | 1               | 2
| **22336** | 42.68             | >3600     | 0               | 6
| 22792     | 100               | 2         | 1               | 0
| 22949     | 100               | 11        | 1               | 0
| 29433     | 92.65             | >3600     | 0               | 15
| 30509     | 0                 | 893       | 2               | 97
| 30532     | 21.44 -> 38.72    | 335       | 2               | 28
| 30654     | 0                 | >3600     | 0               | 400 (hard limit)
|
| -- colored:
| 510       | 83.76             | 26        | 2               | 6
| **672**   | 76.38 -> 76.39    | >3600     | 0               | 42
| 1677      | 90.60 -> 100      | 12        | 1               | 2
| 2166      | 69.67 -> 100      | 24        | 1               | 2
| **2498**  | 90.68             | >3600/1309| 0/2             | 26/63
| **2684**  | 95.33             | 230       | 2               | 11
| 2798      | 100               | 9         | 1               | 0
| 3073      | 96.41 -> 100      | 8         | 1               | 2
| 3085      | 88.91 -> 88.98    | >3600     | 0               | 8
| 3114      | 81.61             | 186       | 2               | 15
| **3149**  | 78.62             | >3600     | 0               | 21
| 3263      | 97.23 -> 100      | 5         | 1               | 1
| **3620**  | 85.81             | >3600/16  | 0/2             | 17/25
| 3883      | 93.46             | 27        | 4               | 10
| **4364**  | 97.42 -> 100      | 6         | 1               | 1
| **4445**  | 93.98             | 73        | 2               | 5
| 4518      | 94.73 -> 100      | 4         | 1               | 3
| 4581      | 76 -> 100         | 3         | 1               | 1
| **4809**  | 89.58 -> 89.70    | 39        | 2               | 5
| 4905      | 93.92 -> 100      | 7         | 1               | 1
| 5260      | 93.75             | 1         | 2               | 13
| 5380      | 87.32             | 22        | 3               | 36
| 5381      | 98.44             | 5         | 2               | 4
| 6406      | 92.74 -> 100      | 9         | 1               | 1
| 6524      | 97.41 -> 97.63    | 69        | 2               | 8
| 6621      | 84.53             | 9         | 2               | 16
| 6704      | 92.97             | 8         | 2               | 12
| 7333      | 92.22             | 5         | 2               | 10
| 7541      | 97.13             | >3600     | 0               | 9
| 7778      | 85.98             | 85        | 2               | 17
| 7817      | 85.19             | 50        | 2               | 29
| 8252      | 92.53             | 5         | 5               | 10
| 8825      | 89.73             | 16        | 2               | 21
| 9616      | 92.13             | 9         | 3               | 24
| 9786      | 63.35             | >3600     | 0               | 7
| 9891      | 72.84 -> 73.50    | 30        | 8               | 2
| 10055     | 96.48 -> 100      | 9         | 1               | 1
| 10282     | 91.63             | 4         | 2               | 5
| 10448     | 70.56             | 91        | 2               | 36
| 10585     | 85.85             | >3600     | 0               | 14
| 11026     | 80.96 -> 80.97    | 354       | 2               | 64
| 11058     | 92.89             | 2022/183  | 2               | 13/78
| 11341     | 72.44             | 12        | 3               | 65
| 11518     | 98.31 -> 100      | 13        | 1               | 2
| 11546     | 60.69             | 8         | 2               | 16
| 11884     | 68.22             | 24        | 2               | 39
| 12831     | 90.75             | >3600/380 | 0/2             | 12/31
| 14164     | 97.42 -> 100      | 6         | 1               | 2
| 14579     | 94.12 -> 100      | 5         | 1               | 1
| 14717     | 84.00             | 35        | 2               | 23
| 15101     | 87.62             | 6         | 2               | 38
| 15225     | 97.85 -> 100      | 27        | 1               | 3
| 15430     | 69.38 -> 100      | 9         | 1               | 2
| 16393     | 78.72             | 16        | 2               | 9
| 16552     | 96.40             | 6/46      | 4/3             | 17/60
| 16712     | 89                | 7         | 2               | 18
| 16766     | 96.04             | 8         | 2               | 5
| 16820     | 88.91             | 27        | 2               | 20
| 16838     | 68.92 -> 69.08    | >3600     | 0               | 13
| 16878     | 86.22             | >3600     | 0               | 13
| 16905     | 96.69             | 558       | 2               | 0
| 17045     | 95.96 -> 95.99    | >3600/116 | 0/4             | 8/22
| 17459     | 92.98             | 36        | 2               | 35
| 18290     | 37.5              | >3600/101 | 0/2             | 17/108
| 19793     | 81.78 -> 100      | 14        | 1               | 1
| 20607     | 95.96             | 3         | 6               | 2
| 21643     | 97.77 -> 100      | 17        | 1               | 3
| 22027     | 99.06             | 21/200    | 2/4             | 16/52
| 22169     | 86.06             | 57        | 2,4             | 31
| 22284     | 89.82 -> 90.61    | 50        | 2               | 5
| 22569     | 98.64 -> 100      | 7         | 1               | 2
| 25158     | 85.70 -> 85.81    | >3600     | 0               | 8
| 29436     | 84.40 -> 84.41    | >3600     | 0               | 8
| 29826     | 86.72 -> 86.80    | 561       | 2               | 16
| 30640     | 96.79             | >3600     | 0               | 9


### The same, using PyPy interpreter (>50 sec)

| #         |  Solution rate, % | Time, sec | Solutions found | Search depth |
|-----------|:-----------------:|----------:|----------------:|-------------:|
| -- black and white:
| **2556**  | 92.72 -> 94.50    | 282       | 3               | 10
| **2712**  | 54.78 -> 55.05    | 79        | 2               | 7
| **6574**  | 29.6 -> 100       | 112       | 1               | 8
| **8098**  | 0 -> 38.50        | 269       | 1               | 8
| **9892**  | 33.30 -> 33.35    | >3600     | 0               | 17
| **12548** | 18.56             | >3600     | 0               | 13
| 13480     | 12.68             | >3600     | 0               | 13
| 16900     | 48.06 -> 48.86    | >3600     | 0               | 12
| **18297** | 13.49 -> 14.15    | 485       | 2               | 8
| **22336** | 42.68 -> 42.72    | >3600     | 0               | 6
| 29433     | 92.65             | 6         | 2               | 21
| 30509     | 0                 | 324       | 2               | 97
| 30532     | 21.44 -> 38.72    | 98        | 2               | 28
| 30654     | 0                 | >3600     | 0               | 157
|
| -- colored:
| **672**   | 76.38 -> 76.39    | >3600     | 0               | 43
| **2498**  | 90.68             | 2365      | 3               | 27
| **2684**  | 95.33             | 112       | 2               | 11
| 3085      | 88.91 -> 88.98    | >3600     | 0               | 9
| 3114      | 81.61             | 100       | 2               | 15
| **3149**  | 78.62             | >3600     | 0               | 22
| **3620**  | 85.81             | >3600     | 0               | 18
| 7541      | 97.13             | 3482      | 2               | 22
| 9786      | 63.35             | >3600     | 0               | 7
| 10585     | 85.85             | >3600     | 0               | 16
| 11026     | 80.96 -> 80.97    | 170       | 2               | 63
| 11058     | 92.89             | 1064      | 2               | 13
| 12831     | 90.75             | >3600     | 0               | 12
| 16552     | 96.40             | >3600     | 0               | 14
| 16838     | 68.92 -> 69.08    | >3600     | 0               | 13
| 16878     | 86.22             | >3600     | 0               | 14
| 16905     | 96.69             | 216       | 2               | 0
| 17045     | 95.96 -> 96.01    | >3600     | 0               | 8
| 18290     | 37                | >3600     | 0               | 22
| 25158     | 85.70 -> 85.81    | >3600     | 0               | 8
| 29436     | 84.40 -> 84.41    | >3600     | 0               | 8
| 29826     | 86.72 -> 86.80    | 261       | 2               | 16
| 30640     | 96.79             | >3600     | 0               | 9


**Bold** puzzles are from http://webpbn.com/survey/ (_italic_ puzzles are mentioned there too).
The slash ('/') in the results signifies that the results are pretty
different with previous major algorithm change.
